import com.vmware.vim.*;
import com.vmware.apputils.*;
import com.vmware.apputils.vim.*;

import java.util.*;
import java.net.URL;
import java.lang.reflect.Method;
import java.rmi.RemoteException;


/**
*<pre>
*Ex - 4 This sample displays the performance measurements for active, swapin, swapout and ballooned memory coutners 
 from the current time at the console.
*
*<b>Parameters:</b>
*vmname      [required]:name of the virtual machine
*
*</pre>
*/



public class VMMemoryCounters {
   
   public ManagedObjectReference _svcRef;
   public VimServiceLocator _locator;
   public VimPortType _service;
   public ServiceContent _sic;
   private ManagedObjectReference _virtualMachine = null;
   private ManagedObjectReference  _perfManager;


   public void createServiceRef() throws Exception {
      _svcRef = new ManagedObjectReference();
      _svcRef.setType("ServiceInstance");
      _svcRef.set_value("ServiceInstance");
   }

   public void connectAndLogin(String[] args) throws Exception {
      String url = args[0];
      String username = args[1];
      String password = args[2];
      
      createServiceRef();
      _locator = new VimServiceLocator();
      _locator.setMaintainSession(true);
      _service = _locator.getVimPort(new URL(url));
      _sic = _service.retrieveServiceContent(_svcRef);
      if (_sic.getSessionManager() != null) {
         _service.login(_sic.getSessionManager(), username, password, null);
      }
   }

   public void disconnect() throws Exception {
      if (_service != null) {
         _service.logout(_sic.getSessionManager());
         _service = null;
         _sic = null;
      }
   }
   

 private void findMemCounters(String vmname) throws Exception {
   	  try {
         String[][] typeinfo = new String[][] { new String[] { "VirtualMachine", "name" },}; 
         _virtualMachine =  getMOR(typeinfo, vmname); 
               
	      if(_virtualMachine!=null) {
	      	 System.out.println("Stats for Virtual Machine " + vmname + " are:");
	      	 System.out.println();
	         ManagedObjectReference pmRef 
	            = _sic.getPerfManager();
	         PerfCounterInfo[] cInfo 
	            = (PerfCounterInfo[])getDynamicProperty(pmRef,"perfCounter");
	         List vmMemCounters = new ArrayList();
	         for(int i=0; i<cInfo.length; ++i) {
	            if("mem".equalsIgnoreCase(cInfo[i].getGroupInfo().getKey())) {
				   if (("active".equalsIgnoreCase(cInfo[i].getNameInfo().getKey()))&&("average".equalsIgnoreCase(cInfo[i].getRollupType().toString()))) {
                      vmMemCounters.add(cInfo[i]);
				    }
				   if (("swapin".equalsIgnoreCase(cInfo[i].getNameInfo().getKey()))&&("average".equalsIgnoreCase(cInfo[i].getRollupType().toString()))) {
                      vmMemCounters.add(cInfo[i]);
				    }
				   if (("swapout".equalsIgnoreCase(cInfo[i].getNameInfo().getKey()))&&("average".equalsIgnoreCase(cInfo[i].getRollupType().toString()))) {
                      vmMemCounters.add(cInfo[i]);
				   }
				   if (("vmmemctl".equalsIgnoreCase(cInfo[i].getNameInfo().getKey()))&&("average".equalsIgnoreCase(cInfo[i].getRollupType().toString()))) {
                      vmMemCounters.add(cInfo[i]);
				   }
                }//end-if-mem
			 }
	         Map counters = new HashMap();         
	            
	            int intC=0;
	            for(Iterator it = vmMemCounters.iterator(); it.hasNext();) {
	               PerfCounterInfo pcInfo = (PerfCounterInfo)it.next();
	               
	            }
	            
	            for(int index=0;index<cInfo.length;++index) {
	               PerfCounterInfo pcInfo = (PerfCounterInfo)cInfo[index];
	               counters.put(new Integer(pcInfo.getKey()), pcInfo);
	              }
	            
	         PerfMetricId[] aMetrics = _service.queryAvailablePerfMetric(pmRef, 
                                                        _virtualMachine, 
                                                        null, 
                                                        null, 
                                                        new Integer(20));
                                                       
	         ArrayList mMetrics = new ArrayList();
	         if(aMetrics != null) {
	            for(int avIndex=0; avIndex<aMetrics.length; ++avIndex) {
	             for(int cpuIndex=0; cpuIndex<vmMemCounters.size(); ++cpuIndex)	{
	               PerfCounterInfo pcInfo = (PerfCounterInfo)vmMemCounters.get(cpuIndex);
	               if(pcInfo.getKey()==((int)(aMetrics[avIndex].getCounterId()))) {
	                  mMetrics.add(aMetrics[avIndex]);
	               }
	           }
	            }
	         }  
	         
	              
	         monitorPerformance(pmRef, _virtualMachine, mMetrics, counters);
	      }
	      else {
	         System.out.println("Virtual Machine " + vmname + " not found");
	      }
      } catch( Exception e ) {
         System.out.println(e.getMessage());
         e.printStackTrace();
      }
   }
   
   void monitorPerformance(ManagedObjectReference pmRef,
                           ManagedObjectReference vmRef, 
                           ArrayList mMetrics, 
                           Map counters) throws Exception {
      PerfMetricId[] metricIds 
         = (PerfMetricId[])mMetrics.toArray(new PerfMetricId[0]);
      PerfQuerySpec qSpec = new PerfQuerySpec();
      qSpec.setEntity(vmRef);
      qSpec.setFormat("csv");
      qSpec.setMaxSample(new Integer(1));
      qSpec.setMetricId(metricIds);
      qSpec.setIntervalId(new Integer(20));
     
      PerfQuerySpec[] qSpecs = new PerfQuerySpec[] {qSpec};      
            
         PerfEntityMetricBase[] pValues 
            = _service.queryPerf(pmRef,qSpecs);
         if(pValues != null) displayValues(pValues, counters);         
         
   }

   void displayValues(PerfEntityMetricBase[] values, Map counters) {
   
      for(int i=0; i<values.length; ++i) {
      	
      	PerfEntityMetricCSV pms = (PerfEntityMetricCSV)values[i];
        PerfMetricSeriesCSV[] vals = pms.getValue();
        
		if (vals != null) {
		  for(int vi=0; vi<vals.length; vi++){
		  	 
		  	 PerfCounterInfo pci 
               = (PerfCounterInfo)counters.get(
                    new Integer(vals[vi].getId().getCounterId())
                  );
            if(pci != null){
	             System.out.println(pci.getNameInfo().getSummary());;
	           	 System.out.println("Counter : " + pci.getNameInfo().getLabel());
             }
           	 PerfMetricSeriesCSV pmCSV = vals[vi];
           	 System.out.println("Instance: " + vals[vi].getId().getInstance());
           	 System.out.println("Description: " + pci.getNameInfo().getSummary());
           	 System.out.println("Units: " + pci.getUnitInfo().getLabel());
             System.out.println("Sample info: " + pms.getSampleInfoCSV());
             System.out.println("Value : " + pmCSV.getValue());
             System.out.println();
	        }
          } else {
          	System.out.println("Perf Counters could not be fetched");
          }
      	
      }
   }
   
   public ManagedObjectReference getMOR(String[][] typeinfo, String name) throws Exception{
      try {
         ManagedObjectReference useroot = _sic.getRootFolder();
         SelectionSpec[] selectionSpecs = null;         
         selectionSpecs = buildFullTraversal();
         
         PropertySpec[] propspecary = buildPropertySpecArray(typeinfo);
         PropertyFilterSpec spec = 
         new PropertyFilterSpec(null,null,
            propspecary,
            new ObjectSpec[] { 
               new ObjectSpec(null,null,
                  useroot,
                  Boolean.FALSE,
                  selectionSpecs
               ) 
            }
         );
         ObjectContent[] retoc =
           _service.retrieveProperties(
           _sic.getPropertyCollector(),
           new PropertyFilterSpec[] { spec }
        );
        ManagedObjectReference mor = null;
        for(int i=0;i<retoc.length;i++) {
           ManagedObjectReference obj = retoc[i].getObj();
           String objName = (String)getDynamicProperty(obj,"name");
           if(objName.equalsIgnoreCase(name)) {
              mor = obj;
           }
        }
        if(mor == null) {
           System.out.println("VM not found");
           System.exit(1);
        }
        return mor;
     } catch( Exception e ) {
        System.out.println(e.getMessage());
        e.printStackTrace();
        return null;
     }
   }
   
   public SelectionSpec [] buildFullTraversal() {
     
      // Recurse through all ResourcePools
      TraversalSpec rpToRp = new TraversalSpec(
            null, null, null,
            "ResourcePool", 
            "resourcePool", 
            Boolean.TRUE, 
            new SelectionSpec[] {new SelectionSpec(null, null, "rpToRp"),
                                 new SelectionSpec(null, null, "rpToVm")}
      );
      rpToRp.setName("rpToRp");
      
      // Recurse through all ResourcePools	  
      TraversalSpec rpToVm = new TraversalSpec(null,null,null,
            "ResourcePool", 
            "vm", 
            Boolean.FALSE, 
            new SelectionSpec[] {}
      );
      rpToVm.setName("rpToVm");
      
      // Traversal through ResourcePool branch
      TraversalSpec crToRp = new TraversalSpec(
            null, null, null,
            "ComputeResource", 
            "resourcePool", 
            Boolean.TRUE, 
            new SelectionSpec[] {new SelectionSpec(null, null, "rpToRp"),
                                 new SelectionSpec(null, null, "rpToVm")}
      );
      crToRp.setName("crToRp");
      
      // Traversal through host branch
      TraversalSpec crToH = new TraversalSpec(
            null, null, null,
            "ComputeResource", 
            "host", 
            Boolean.TRUE, 
            new SelectionSpec[] {}
      );
      crToH.setName("crToH");
     // Traversal through hostFolder branch
      TraversalSpec dcToHf = new TraversalSpec(
            null, null, null,
            "Datacenter", 
            "hostFolder", 
            Boolean.TRUE, 
            new SelectionSpec[] {               
               new SelectionSpec(null, null, "visitFolders")
            }
      );
      dcToHf.setName("dcToHf");
      
      // Traversal through vmFolder branch
      TraversalSpec dcToVmf = new TraversalSpec(
            null, null, null,
            "Datacenter", 
            "vmFolder", 
            Boolean.TRUE, 
            new SelectionSpec[] {
               new SelectionSpec(null, null, "visitFolders")               
            }
      );
      dcToVmf.setName("dcToVmf");   
      
      // Recurse through all Hosts
      TraversalSpec HToVm = new TraversalSpec(null,null,null,
            "HostSystem", 
            "vm", 
            Boolean.FALSE, 
            new SelectionSpec[] {new SelectionSpec(null, null, "visitFolders")}
      );
      HToVm.setName("HToVm");
      
      // Recurse thriugh the folders
      TraversalSpec visitFolders = 
         new TraversalSpec(
               null, null, null,
               "Folder", 
               "childEntity", 
               Boolean.FALSE, 
               new SelectionSpec[] {
                  new SelectionSpec(null, null, "visitFolders"),
                  new SelectionSpec(null, null, "dcToHf"),
                  new SelectionSpec(null, null, "dcToVmf"),
                  new SelectionSpec(null, null, "crToH"),
                  new SelectionSpec(null, null, "crToRp"),
                  new SelectionSpec(null, null, "HToVm"),
                  new SelectionSpec(null, null, "rpToVm"),
               }
         );
      visitFolders.setName("visitFolders");
      return new SelectionSpec [] {visitFolders,dcToVmf,dcToHf,crToH,crToRp,rpToRp,HToVm,rpToVm};
   }
   
   public PropertySpec[] buildPropertySpecArray(
      String[][] typeinfo
   ) {
      // Eliminate duplicates
      HashMap tInfo = new HashMap();
      for(int ti = 0; ti < typeinfo.length; ++ti) {
         Set props = (Set)tInfo.get(typeinfo[ti][0]);
         if(props == null) {
            props = new HashSet();
            tInfo.put(typeinfo[ti][0], props);
         }
         boolean typeSkipped = false;
         for(int pi=0; pi<typeinfo[ti].length; ++pi) {
            String prop = typeinfo[ti][pi];
            if(typeSkipped) {
               props.add(prop);
            } else {
               typeSkipped = true;
            }
         }
      }

      // Create PropertySpecs
      ArrayList pSpecs = new ArrayList();
      for(Iterator ki = tInfo.keySet().iterator(); ki.hasNext();) {
         String type = (String)ki.next();
         PropertySpec pSpec = new PropertySpec();
         Set props = (Set)tInfo.get(type);
         pSpec.setType(type);
         pSpec.setAll(props.isEmpty()?Boolean.TRUE:Boolean.FALSE);
         pSpec.setPathSet(new String[props.size()]);
         int index = 0;
         for(Iterator pi = props.iterator(); pi.hasNext();) {
            String prop = (String)pi.next();
            pSpec.setPathSet(index++, prop);
         }
         pSpecs.add(pSpec);
      }
      
      return (PropertySpec[])pSpecs.toArray(new PropertySpec[0]);
   }
   
      public ObjectContent[] getObjectProperties(
      ManagedObjectReference collector,
      ManagedObjectReference mobj, String[] properties
   ) throws Exception {
      if (mobj == null) {
         return null;
      }
      ManagedObjectReference usecoll = collector;
      if (usecoll == null) {
         usecoll = _sic.getPropertyCollector();
      }
      PropertyFilterSpec spec = new PropertyFilterSpec();
      spec.setPropSet(new PropertySpec[] { new PropertySpec() });
      spec.getPropSet(0).setAll(new Boolean (properties == null || properties.length == 0));
      spec.getPropSet(0).setType(mobj.getType());
      spec.getPropSet(0).setPathSet(properties);
      spec.setObjectSet(new ObjectSpec[] { new ObjectSpec() });
      spec.getObjectSet(0).setObj(mobj);
      spec.getObjectSet(0).setSkip(Boolean.FALSE);
      return _service.retrieveProperties(usecoll, new PropertyFilterSpec[] { spec });
   }

   public Object getDynamicProperty(
         ManagedObjectReference mor,
         String propertyName
   ) throws Exception {
      ObjectContent[] objContent = getObjectProperties(null, mor,
            new String[] { propertyName });
      
      Object propertyValue = null;
      if (objContent != null) {
         DynamicProperty[] dynamicProperty = objContent[0].getPropSet();
         if (dynamicProperty != null) {
            Object dynamicPropertyVal = dynamicProperty[0].getVal();
            String dynamicPropertyName = dynamicPropertyVal.getClass().getName();
            if (dynamicPropertyName.indexOf("ArrayOf") != -1) {
               String methodName = dynamicPropertyName.substring(
                     dynamicPropertyName.indexOf("ArrayOf")
                     + "ArrayOf".length(), 
                     dynamicPropertyName.length());              
               if (methodExists(dynamicPropertyVal,
                     "get" + methodName, null)) {
                  methodName = "get" + methodName;
               } else {
                  
                  methodName = "get_" + 
                  methodName.toLowerCase();
               }
               Method getMorMethod = dynamicPropertyVal.getClass().
               getDeclaredMethod(methodName,  (Class[])null);
               propertyValue = getMorMethod.invoke(dynamicPropertyVal, (Object[])null);
            } else if (dynamicPropertyVal.getClass().isArray()) {               
               propertyValue = dynamicPropertyVal;
            } else {
               propertyValue = dynamicPropertyVal;
            }
         }
      }
      return propertyValue;
   }
   
   boolean methodExists(Object obj, 
         String methodName, 
         Class[] parameterTypes) {
      boolean exists = false;
      try {
         Method method = obj.getClass().getMethod(methodName, parameterTypes);
         if (method != null) {
            exists = true;
         }
      } catch(Exception e){}
      return exists;
   }
   

   public static void main(String[] args) throws Exception {
	  System.setProperty("axis.socketSecureFactory", 
                       "org.apache.axis.components.net.SunFakeTrustSocketFactory");	
      VMMemoryCounters obj = new VMMemoryCounters();
      obj.connectAndLogin(args);    
      obj.findMemCounters(args[3]);
      obj.disconnect(); 
	  
	  
   }
}